@@ -1,7 +1,5 @@
 import logging
 
-import pypot.utils.pypot_time as time
-
 from ..primitive.manager import PrimitiveManager
 
 
@@ -18,13 +16,15 @@ class Robot(object):
     This class also provides a generic motors accessor in order to (more or less) easily extends this class to other types of motor.
 
         """
-    def __init__(self, motor_controllers=[], sensor_controllers=[]):
+    def __init__(self, motor_controllers=[], sensor_controllers=[], sync=True):
         """
         :param list motor_controllers: motors controllers to attach to the robot
         :param list sensor_controllers: sensors controllers to attach to the robot
+        :param bool sync: choose if automatically starts the synchronization loops
 
         """
         self._motors = []
+        self._sensors = []
         self.alias = []
 
         self._controllers = sensor_controllers + motor_controllers
@@ -39,16 +39,19 @@ class Robot(object):
             for s in controller.sensors:
                 setattr(self, s.name, s)
 
+            self._sensors.extend(controller.sensors)
+
         self._attached_primitives = {}
         self._primitive_manager = PrimitiveManager(self.motors)
 
         self._syncing = False
-        self.start_sync()
+        if sync:
+            self.start_sync()
 
     def close(self):
         """ Cleans the robot by stopping synchronization and all controllers."""
         self.stop_sync()
-        [c.io.close() for c in self._controllers]
+        [c.io.close() for c in self._controllers if c.io is not None]
 
     def __repr__(self):
         return '<Robot motors={}>'.format(self.motors)
@@ -61,6 +64,7 @@ class Robot(object):
         [c.start() for c in self._controllers]
         [c.wait_to_start() for c in self._controllers]
         self._primitive_manager.start()
+        self._primitive_manager._running.wait()
 
         self._syncing = True
 
@@ -73,7 +77,9 @@ class Robot(object):
 
         if self._primitive_manager.running:
             self._primitive_manager.stop()
+
         [c.stop() for c in self._controllers]
+        [s.close() for s in self.sensors if hasattr(s, 'close')]
 
         self._syncing = False
 
@@ -91,6 +97,11 @@ class Robot(object):
         """ Returns all the motors attached to the robot. """
         return self._motors
 
+    @property
+    def sensors(self):
+        """ Returns all the sensors attached to the robot. """
+        return self._sensors
+
     @property
     def active_primitives(self):
         """ Returns all the primitives currently running on the robot. """
@@ -112,22 +123,22 @@ class Robot(object):
         for m in self.motors:
             m.compliant = is_compliant
 
-    def goto_position(self, position_for_motors, duration, wait=False):
+    def goto_position(self, position_for_motors, duration, control=None, wait=False):
         """ Moves a subset of the motors to a position within a specific duration.
 
             :param dict position_for_motors: which motors you want to move {motor_name: pos, motor_name: pos,...}
             :param float duration: duration of the move
+            :param str control: control type ('dummy', 'minjerk')
             :param bool wait: whether or not to wait for the end of the move
 
             .. note::In case of dynamixel motors, the speed is automatically adjusted so the goal position is reached after the chosen duration.
 
             """
-        for motor_name, position in position_for_motors.iteritems():
-            m = getattr(self, motor_name)
-            m.goto_position(position, duration)
+        for i, (motor_name, position) in enumerate(position_for_motors.iteritems()):
+            w = False if i < len(position_for_motors) - 1 else wait
 
-        if wait:
-            time.sleep(duration)
+            m = getattr(self, motor_name)
+            m.goto_position(position, duration, control, wait=w)
 
     def power_up(self):
         """ Changes all settings to guarantee the motors will be used at their maximum power. """
@@ -168,6 +179,9 @@ class Robot(object):
                 'angle_limit': m.angle_limit,
             }
 
+            if m.angle_limit == (0, 0):
+                config['motors']['wheel_mode'] = True
+
         config['motorgroups'] = {}
 
         return config
