@@ -2,6 +2,7 @@ import json
 import logging
 
 from functools import partial
+from collections import OrderedDict
 
 from .io import (VrepIO, close_all_connections,
                  VrepIOError, VrepConnectionError)
@@ -24,9 +25,16 @@ class vrep_time():
     def __init__(self, vrep_io):
         self.io = vrep_io
 
-    def get_time(self):
+    def get_time(self, trial=0):
         t = self.io.get_simulation_current_time()
 
+        if t == 0:
+            sys_time.sleep(.5)
+            return self.get_time(trial + 1)
+
+        if trial > 10:
+            raise EnvironmentError('Could not get current simulation time. Make sure the V-REP simulation is running. And that you have added the "time" child script to your scene.')
+
         return t
 
     def sleep(self, t):
@@ -80,13 +88,7 @@ def from_vrep(config, vrep_host='127.0.0.1', vrep_port=19997, scene=None,
 
     if isinstance(config, basestring):
         with open(config) as f:
-            config = json.load(f)
-
-    # The URDF uses the offset as the 0 for the motors equivalent
-    # so we set all the offsets to 0
-    config = dict(config)
-    for m in config['motors'].itervalues():
-        m['offset'] = 0.0
+            config = json.load(f, object_pairs_hook=OrderedDict)
 
     motors = [motor_from_confignode(config, name)
               for name in config['motors'].keys()]
@@ -109,24 +111,19 @@ def from_vrep(config, vrep_host='127.0.0.1', vrep_port=19997, scene=None,
     robot = Robot(motor_controllers=[vc],
                   sensor_controllers=sensor_controllers)
 
+    for m in robot.motors:
+        m.goto_behavior = 'minjerk'
+
     init_pos = {m: m.goal_position for m in robot.motors}
 
     make_alias(config, robot)
 
-    def reset(robot):
+    def start_simu():
+        vrep_io.start_simulation()
+
         for m, p in init_pos.iteritems():
             m.goal_position = p
 
-        if tracked_objects:
-            vot.stop()
-
-        if tracked_collisions:
-            vct.stop()
-
-        vc.stop()
-
-        vrep_io.restart_simulation()
-
         vc.start()
 
         if tracked_objects:
@@ -138,7 +135,24 @@ def from_vrep(config, vrep_host='127.0.0.1', vrep_port=19997, scene=None,
         while vrep_io.get_simulation_current_time() < 1.:
             sys_time.sleep(0.1)
 
-    robot.reset_simulation = lambda: reset(robot)
+    def stop_simu():
+        if tracked_objects:
+            vot.stop()
+
+        if tracked_collisions:
+            vct.stop()
+
+        vc.stop()
+        vrep_io.stop_simulation()
+
+    def reset_simu():
+        stop_simu()
+        sys_time.sleep(0.5)
+        start_simu()
+
+    robot.start_simulation = start_simu
+    robot.stop_simulation = stop_simu
+    robot.reset_simulation = reset_simu
 
     def current_simulation_time(robot):
         return robot._controllers[0].io.get_simulation_current_time()
@@ -148,4 +162,8 @@ def from_vrep(config, vrep_host='127.0.0.1', vrep_port=19997, scene=None,
         return vrep_io.get_object_position(object, relative_to_object)
     Robot.get_object_position = partial(get_object_position, robot)
 
+    def get_object_orientation(robot, object, relative_to_object=None):
+        return vrep_io.get_object_orientation(object, relative_to_object)
+    Robot.get_object_orientation = partial(get_object_orientation, robot)
+
     return robot
