@@ -1,7 +1,10 @@
 from numpy import rad2deg, deg2rad
+from collections import deque
 
 from ..robot.controller import MotorsController, SensorsController
+from ..dynamixel.conversion import torque_max
 from ..robot.sensor import Sensor
+from .io import remote_api
 
 
 class VrepController(MotorsController):
@@ -26,22 +29,55 @@ class VrepController(MotorsController):
         """ Setups the controller by reading/setting position for all motors. """
         self._init_vrep_streaming()
 
+        # Init lifo for temperature spoofing
+        for m in self.motors:
+            m.__dict__['_load_fifo'] = deque(200 * [1], maxlen=200)
+
+        self.update()
+
     def update(self):
         """ Synchronization update loop.
 
         At each update all motor position are read from vrep and set to the motors. The motors target position are also send to v-rep.
 
         """
+        # Read all the angle limits
+        h, _, l, _ = self.io.call_remote_api('simxGetObjectGroupData',
+                                             remote_api.sim_object_joint_type,
+                                             16,
+                                             streaming=True)
+        limits4handle = {hh: (ll, lr) for hh, ll, lr in zip(h, l[::2], l[1::2])}
+
         for m in self.motors:
+            tmax = torque_max[m.model]
+
             # Read values from V-REP and set them to the Motor
             p = round(
                 rad2deg(self.io.get_motor_position(motor_name=m.name)), 1)
             m.__dict__['present_position'] = p
 
+            l = 100. * self.io.get_motor_force(motor_name=m.name) / tmax
+            m.__dict__['present_load'] = l
+
+            m.__dict__['_load_fifo'].append(abs(l))
+            m.__dict__['present_temperature'] = 25 + \
+                round(2.5 * sum(m.__dict__['_load_fifo']) / len(m.__dict__['_load_fifo']), 1)
+
+            ll, lr = limits4handle[self.io._object_handles[m.name]]
+            m.__dict__['lower_limit'] = rad2deg(ll)
+            m.__dict__['upper_limit'] = rad2deg(ll) + rad2deg(lr)
+
             # Send new values from Motor to V-REP
             p = deg2rad(round(m.__dict__['goal_position'], 1))
             self.io.set_motor_position(motor_name=m.name, position=p)
 
+            t = m.__dict__['torque_limit'] * tmax / 100.
+
+            if m.__dict__['compliant']:
+                t = 0.
+
+            self.io.set_motor_force(motor_name=m.name, force=t)
+
     def _init_vrep_streaming(self):
         # While the code below may look redundant and that
         # it could be simplified. It is written as such to
@@ -51,10 +87,11 @@ class VrepController(MotorsController):
 
         # Prepare streaming for getting position for each motor
         for m in self.motors:
-            self.io.call_remote_api('simxGetJointPosition',
-                                    self.io.get_object_handle(m.name),
-                                    streaming=True,
-                                    _force=True)
+            for vrep_call in ['simxGetJointPosition', 'simxGetJointForce']:
+                self.io.call_remote_api(vrep_call,
+                                        self.io.get_object_handle(m.name),
+                                        streaming=True,
+                                        _force=True)
 
         # Now actually retrieves all values
         pos = [self.io.get_motor_position(m.name) for m in self.motors]
@@ -67,11 +104,30 @@ class VrepController(MotorsController):
                                     sending=True,
                                     _force=True)
 
+        for m in self.motors:
+            self.io.call_remote_api('simxSetJointForce',
+                                    self.io.get_object_handle(m.name),
+                                    torque_max[m.model],
+                                    sending=True,
+                                    _force=True)
+
+        # Prepare streaming for the angle limit
+        self.io.call_remote_api('simxGetObjectGroupData',
+                                remote_api.sim_object_joint_type,
+                                16,
+                                streaming=True,
+                                _force=True)
+
         # And actually affect them
         for m, p in zip(self.motors, pos):
             self.io.set_motor_position(m.name, p)
             m.__dict__['goal_position'] = rad2deg(p)
 
+        for m in self.motors:
+            self.io.set_motor_force(m.name, torque_max[m.model])
+            m.__dict__['torque_limit'] = 100.
+            m.__dict__['compliant'] = False
+
 
 class VrepObjectTracker(SensorsController):
 
